import { run, bench, group } from 'mitata'
import { randomBytes } from 'node:crypto'

import ZVToken from '../src'
import jwt from 'jsonwebtoken'
import { SignJWT, jwtVerify } from 'jose'
import { createSigner, createVerifier } from 'fast-jwt'

const SECRET = randomBytes(16).toString('hex')
const SECRET_UINT = new TextEncoder().encode(SECRET)

const zvt = new ZVToken({ secret: SECRET, structures: ['sub', 'iss', 'exp', 'iat'] })
const fastJwtSigner = createSigner({ key: SECRET })

const data = {
    sub: '123',
    iss: 'my app',
    exp: 2147483647,
    iat: Date.now() / 1000 | 0
}

group('sign', () => {
    bench('zv-token', () => zvt.sign(data))
    bench('jsonwebtoken', () => jwt.sign(data, SECRET))
    bench('jose', () => {
        const jose = new SignJWT(data).setProtectedHeader({ alg: 'HS256' })
        jose.sign(SECRET_UINT)
    })
    bench('fast-jwt', () => fastJwtSigner(data))
})

const zvt_token = zvt.sign(data)
const jwt_token = jwt.sign(data, SECRET)

const jose = new SignJWT(data).setProtectedHeader({ alg: 'HS256' })
const jose_token = await jose.sign(SECRET_UINT)

const fast_jwt_token = fastJwtSigner(data)
const fastJwtVerifier = createVerifier({ key: SECRET })

group('verify', () => {
    bench('zv-token', () => zvt.verify(zvt_token))
    bench('jsonwebtoken', () => jwt.verify(jwt_token, SECRET))
    bench('jose', () => {jwtVerify(jose_token, SECRET_UINT)})
    bench('fast-jwt', () => fastJwtVerifier(fast_jwt_token))
})

await run()