import { Packr } from 'msgpackr'

interface ZVTOptions {
    secret: string
    throwErrors?: boolean
    structures?: string[]
    exp_key?: 'exp' | string
}

class ZVToken<T extends Record<string, any>> {
    packr
    hasher
    secret
    exp_key
    throwErrors

    constructor (opts: ZVTOptions) {
        this.secret = opts.secret
        this.exp_key = opts.exp_key || 'exp'
        this.throwErrors = opts.throwErrors || false
        this.hasher = new Bun.CryptoHasher('sha256')

        this.packr = new Packr({
            structures: [[this.exp_key, ...(opts.structures || [])]]
        })
    }

    sign (payload: T) {
        const p = this.packr.pack(payload)
            .toString('base64')

        this.hasher.update(p + this.secret)
        const sign = this.hasher.digest('base64')

        return `zv.${p}.${sign}`
            .replace(/\//g, '_')
            .replace(/\+/g, '-')
            .replace(/=/g, '')
    }

    error (msg: string) {
        if (this.throwErrors) throw new Error(msg)
        return { status: false, payload: undefined }
    }

    verify (token: string) {
        const [, p, sign] = token
            .replace(/_/g, '/')
            .replace(/-/g, '+')
            .split('.')

        this.hasher.update(p + this.secret)
        const correctSign = this.hasher.digest('base64')
        if (sign !== correctSign.replace(/=/g, '')) {
            return this.error('incorrect sign')
        }

        const payload = this.packr.unpack(Buffer.from(p, 'base64')) as T

        if (this.exp_key in payload) {
            const date = Math.floor(Date.now() / 1000)
            if (date - +payload.expires > 0)
                return this.error('session expired')
        }

        return { status: true, payload }
    }
}

export default ZVToken
